<?php
/**
 * @author dongbeta <zimple@imdong.net>
 * @copyright Copyright @ 2009-2010 Zimple Project
 * @link http://zimple.googlecode.com
 * @license	 http://www.opensource.org/licenses/bsd-license.php
 * @version $Id$
 */
class TokenManager extends CApplicationComponent
{

	/**
	 * create a token
	 * @param string $action
	 * @param unknown_type $params
	 * @param int $duration
	 * @param int $data
	 * @return boolean
	 */
	public function create($action, $params, $duration = 0, $data = null)
	{
		$identityKey = $this->_createIdentityKey($action, $params);
		$this->deleteByIdentityKey($action, $identityKey);
		$tokenKey = $this->createTokenKey($identityKey);
		$expireTime = $duration > 0 ? $duration + time() : 0;
		$token = $this->createToken($action, $identityKey, $tokenKey, $expireTime, serialize($data));
		return is_string($token) ? $token : false;
	}

	/**
	 * validate a token
	 *
	 * @param string $action
	 * @param string $token
	 * @return data|boolean
	 */
	public function validate($action, $token)
	{
		$validate = $this->validateToken($action, $token);
		if($validate === false)
			return false;
		$this->deleteByTokenKey($action, $token);
		return $validate;
	}

	/**
	 * create identity key
	 * @param string $action
	 * @param mixed $params
	 * @return string
	 */
	protected function _createIdentityKey($action, $params)
	{
		return md5($action . serialize($params));
	}

	/**
	 * create token string
	 * @param string $action
	 * @param mixed $params
	 * @return string
	 */
	protected function createTokenKey($identityKey)
	{
		return md5($identityKey . SECRET_KEY . time());
	}

	/**
	 * create a token in database
	 * @param string $name
	 * @param string $identityKey
	 * @param string $tokenKey
	 * @param int $expireTime
	 * @param string $data serialized data
	 * @return boolean
	 */
	protected function createToken($action, $identityKey, $tokenKey, $expireTime, $data = null)
	{
		$token = new Token();
		$token->action = $action;
		$token->identity = $identityKey;
		$token->token = $tokenKey;
		$token->expire_time = $expireTime;
		$token->data = $data;
		if(!$token->save())
		{
			Yii::log("Config item cannot be create: $action :: $identityKey", CLogger::LEVEL_ERROR, 'application.components.token.YDbToken');
			return false;
		}
		return $tokenKey;
	}

	/**
	 * validata a token
	 * @param string $name
	 * @param string $token
	 * @return boolean
	 */
	protected function validateToken($action, $token)
	{
		$criteria = new CDbCriteria();
		$criteria->condition = 'action = :action AND token = :token';
		$criteria->params = array(':action'=>$action, 'token'=>$token);
		$record = Token::model()->find($criteria);
		if(!($record instanceof Token) || $record->token != $token || ($record->expire_time > 0 && $record->expire_time < time()))
			return false;
		return unserialize($record->data);
	}

	/**
	 * delele by token key
	 * @param string $action
	 * @param string $identityKey
	 * @return boolean
	 */
	protected function deleteByIdentityKey($action, $identityKey)
	{
		$criteria = new CDbCriteria();
		$criteria->condition = 'action = :action AND identity = :identity';
		$criteria->params = array(':action'=>$action, ':identity'=>$identityKey);
		if(!Token::model()->deleteAll($criteria))
		{
			Yii::log("Cannot delete token by identityKey: $action :: $identityKey", CLogger::LEVEL_ERROR, 'application.components.token.YDbLimit');
			return false;
		}
		return true;
	}

	/**
	 * delele by token key
	 * @param string $action
	 * @param string $tokenKey
	 * @return boolean
	 */
	protected function deleteByTokenKey($action, $tokenKey)
	{
		$criteria = new CDbCriteria();
		$criteria->condition = 'action = :action AND token = :token';
		$criteria->params = array(':action'=>$action, ':token'=>$tokenKey);
		if(!Token::model()->deleteAll($criteria))
		{
			Yii::log("Cannot delete token by tokenKey: $action :: $tokenKey", CLogger::LEVEL_ERROR, 'application.components.token.YDbLimit');
			return false;
		}
		return true;
	}
}
